package com.ephesoft.dcma.da.property;

import com.ephesoft.dcma.core.common.DCMABusinessException;


/**
 * This is a enum BatchPriority. On the basis of priority we have four kinds of
 * BatchPriority's URGENT, HIGH, MEDIUM and LOW. We can get the BatchPriority
 * for input of priority.
 * 
 * @author Ephesoft
 * @version 1.0
 * @see com.ephesoft.dcma.batch.service.BatchSchemaService
 */
public enum BatchPriority {

	URGENT("Urgent", 1, 25), HIGH("High", 26, 50), MEDIUM("Medium", 51, 75), LOW(
			"Low", 76, 100);

	private String batchPriorityName;
	private Integer lowerLimit;
	private Integer upperLimit;

	/**
	 * @param batchPriorityName
	 *            String
	 * @param lowerLimit
	 *            Integer
	 * @param upperLimit
	 *            Integer
	 */
	BatchPriority(String batchPriorityName, Integer lowerLimit,
			Integer upperLimit) {
		this.batchPriorityName = batchPriorityName;
		this.lowerLimit = lowerLimit;
		this.upperLimit = upperLimit;
	}

	/**
	 * @return the batchPriorityName
	 */
	public String getBatchPriorityName() {
		return batchPriorityName;
	}

	/**
	 * @return the lowerLimit
	 */
	public Integer getLowerLimit() {
		return lowerLimit;
	}

	/**
	 * @return the upperLimit
	 */
	public Integer getUpperLimit() {
		return upperLimit;
	}

	/**
	 * This method will return the BatchPriority on the basis of input priority.
	 * 
	 * @param priority
	 *            int
	 * @return BatchPriority
	 */
	public static BatchPriority getBatchPriority(int priority) {

		BatchPriority batchPriority = null;
		boolean isFound = false;

		for (BatchPriority bP : BatchPriority.values()) {
			isFound = getBatchPriority(priority, bP);
			if (isFound) {
				batchPriority = bP;
				break;
			}
		}

		if (null == batchPriority) {
			String errMsg = "BatchPriority not found for the input priority.";

			throw new DCMABusinessException(errMsg);
		}
		return batchPriority;
	}

	/**
	 * This method will check for data i.e. data is present in the range or not.
	 * 
	 * @param priority
	 *            int
	 * @param batchPriority
	 *            BatchPriority
	 * @return true if data found other wise false.
	 */
	private static boolean getBatchPriority(int priority,
			BatchPriority batchPriority) {

		boolean isFound = false;
		int minRange = batchPriority.getLowerLimit();
		int maxRange = batchPriority.getUpperLimit();

		if (priority <= maxRange && priority >= minRange) {
			isFound = true;
		}

		return isFound;
	}

}
